/**
 * アドオンコンテンツクラス
 */
define([
  "dojo/_base/array",
  "dojo/_base/declare",
  "dojo/_base/event",
  "dojo/_base/lang",
  "dojo/dom",
  "dojo/dom-attr",
  "dojo/dom-class",
  "dojo/dom-construct",
  "dojo/dom-style",
  "dojo/on",
  "dojo/query",
  "dojo/request",
  "dojo/store/Memory",
  "dijit/ConfirmDialog",
  "dijit/form/RadioButton",
  "dijit/registry",
  "wb/PaneContent"
], function(
  array,
  declare,
  baseEvent,
  lang,
  dom,
  domAttr,
  domClass,
  domConstruct,
  domStyle,
  on,
  query,
  request,
  Memory,
  ConfirmDialog,
  RadioButton,
  registry,
  PaneContent
) {
  return declare(PaneContent, {
    constructor: function(initValue) {
      this.title = wagbyDesigner.jfcRes.LABEL_TAB_OPERATION_ADDON; // アドオン
      this.frameName = "Addon";
      this.addonGalleryStore = new Memory();
      this.addonGalleryLastUpdated;
      this.addonDefaultColor = "#f5f5f5";
      this.addonSelectColor = "#f1f1b8";
      this.addonBoderDefaultColor = "snow";
      this.addonBoderSelectColor = "#ffff55";
      this.addonNodeSelectedAttrName = "data-addon-selected";
      this.ADDONICON_ID_PREFIX = "addon_icon_";
      this.ADDONICON_UPDATE_CLASS = "blinkAddon";
      this.radioWatchHandles = [];
      this.setContents();
    },

    setContents: function() {
      this.inherited(arguments);
      this.contents.set("href", "wb/paneTemplate/operations/addon.html?" + wagbyDesigner.RELEASE_TIMESTAMP);
      this.contents.set("preload", true);
      this.createAddonDetailDialog();
      this.createAddonAuthKeyInputDialog();
      this.createConfirmDialog();
      this.setEvents();
    },

    onContentsLoaded: function() {
      this.inherited(arguments);
      this.setAddonGalleryAreaDom();
      this.setButtonEvent();
    },

    setScreenResource: function() {
      this.inherited(arguments);
    },

    setEvents: function() {
      var that = this;
      this.contents.on("Show", function() {
        that.show();
      });
      this.contents.on("Hide", function() {
        that.hide();
      });
    },

    show: function() {
      this.inherited(arguments);
      this.getAddonList();
      this.clearSelectedAddon();
    },

    hide: function() {
      this.inherited(arguments);
    },

    createAddonDetailDialog: function() {
      this.addonDetailDialog = new ConfirmDialog({
        title: wagbyDesigner.jfcRes.TITLE_OPERATION_ADDON_DETAIL_DIALOG,
        style: "max-height: 95%; min-height: 90%; width: 80%;",
        preventCache: true,
      });
      this.setAddonDetailDialogStyle();
    },

    setAddonDetailDialogStyle: function() {
      domStyle.set(this.addonDetailDialog.actionBarNode, {
        borderTop: "1px solid #d3d3d3",
        margin: "20px 5px 0 5px",
        padding: "5px",
        textAlign: "center",
      });
      domStyle.set(this.addonDetailDialog.okButton.domNode, {
        fontSize: "1rem",
        fontWeight: "bold",
      });
      domStyle.set(this.addonDetailDialog.okButton.domNode.childNodes[0], {
        width: "100px",
      });
      domStyle.set(this.addonDetailDialog.cancelButton.domNode, "display", "none");
    },

    createAddonAuthKeyInputDialog: function() {
      var that = this;
      this.addonAuthKeyInputDialog = new ConfirmDialog({
        title: wagbyDesigner.jfcRes.TITLE_OPERATION_ADDON_AUTHKEY_INPUT_DIALOG,
        style: "width: 400px;",
        content: that.getAddonAuthKeyInputDialogContent(),
      });
      this.setAddonAuthKeyInputDialogStyle();
      this.setAddonAuthKeyInputDialogEvent();
    },

    getAddonAuthKeyInputDialogContent: function() {
      return '<form data-dojo-type="dijit/form/Form" data-dojo-id="addonAuthKeyForm">' +
        '<p style="margin: 0;">' + wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_AUTHKEY_REQUIRED + '</p>' +
        '<div style="margin: 5px 0; padding: 10px 0;">' +
        '<label for="jfcAddonAuthKey">' + wagbyDesigner.jfcRes.LABEL_OPERATION_ADDON_AUTHKEY + ':</label>' +
        '<div data-dojo-type="dijit/form/ValidationTextBox" data-dojo-id="jfcAddonAuthKey" id="jfcAddonAuthKey" type="password" autocomplete="new-password" data-dojo-props="required: true" style="width: 20em;"></div>' +
        '</div>' +
        '</form>';
    },

    setAddonAuthKeyInputDialogStyle: function() {
      domStyle.set(this.addonAuthKeyInputDialog.actionBarNode, {
        borderTop: "1px solid #d3d3d3",
        margin: "0 5px",
        padding: "5px",
        textAlign: "right",
      });
      domStyle.set(this.addonAuthKeyInputDialog.okButton.domNode.childNodes[0], {
        width: "60px",
      });
    },

    setAddonAuthKeyInputDialogEvent: function() {
      this.addonAuthKeyInputDialog.okButton.on("click", function(e) {
        if (!addonAuthKeyForm.validate()) {
          return false;
        }
        addonAuthKey.set("value", jfcAddonAuthKey.get("value"));
      });
      var that = this;
      this.addonAuthKeyInputDialog.on("Execute", function() {
        that.checkAddonConflict(addonGalleryForm.getValues());
      });
      this.addonAuthKeyInputDialog.on("Hide", function() {
        jfcAddonAuthKey.reset();
      });
    },

    createConfirmDialog: function() {
      this.confirmDialog = wagbyDesigner.createConfirmDialog({});
    },

    showConfirmDialog: function(args) {
      if (args.message) {
        this.confirmDialog.set("content", args.message);
      }
      this.confirmDialog.setEventCallbackFunc(args);
      this.confirmDialog.show();
    },

    setAddonGalleryAreaDom: function() {
      this.addonGalleryArea = dom.byId("addonGalleryArea");
    },

    setButtonEvent: function() {
      this.setBtnAddonInstallEvent();
      this.setBtnAddonUninstallEvent();
    },

    setBtnAddonInstallEvent: function() {
      var that = this;
      jfcBtnAddonInstall.on("click", function() {
        that.confirmExecute(
          wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_CONFIRM_INSTALL,
          lang.hitch(that, that.checkAddonAuthKey),
          that.confirmDialog.okButton
        );
      });
      this.setBtnAddonInstallDisabled(true);
      this.setBtnAddonUninstallDisabled();
    },

    setBtnAddonUninstallEvent: function() {
      var that = this;
      jfcBtnAddonUninstall.on("click", function() {
        that.confirmExecute(
          wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_CONFIRM_UNINSTALL,
          lang.hitch(that, that.uninstallAddon),
          that.confirmDialog.cancelButton
        );
      });
      this.setBtnAddonInstallDisabled(true);
      this.setBtnAddonUninstallDisabled();
    },

    setBtnAddonInstallDisabled: function(disabled) {
      jfcBtnAddonInstall.set("disabled", disabled);
    },

    setBtnAddonUninstallDisabled: function(addonId) {
      var disabled = true;
      if (addonId && this.installedAddons[addonId]) {
        disabled = false;
      }
      jfcBtnAddonUninstall.set("disabled", disabled);
    },

    getAddonList: function() {
      addonGalleryStandby.show();
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/addons",
        options: {
          handleAs: "json",
        }
      }, function(data) {
        that.setAddonGalleryData(data);
        that.getInstalledAddons();
      }, function(err) {
        addonGalleryStandby.hide();
        return wagbyDesigner.jfcRes.MSG_ERROR_OPERATION_ADDON_GET;
      });
    },

    setAddonGalleryData: function(data) {
      if (this.addonGalleryLastUpdated === data.lastUpdated) {
        return;
      }
      this.addonGalleryLastUpdated = data.lastUpdated;
      this.addonGalleryStore.setData([]);
      for (var i=0,l=data.addons.length; i<l; i++) {
        var addon = data.addons[i];
        this.addonGalleryStore.add({
          id: addon.id,
          name: addon.name,
          authKeyRequired: addon.authKeyRequired,
          summary: addon.summary,
          version: addon.version,
          target: addon.target,
          canInstall: this.canInstallAddon(addon.target),
          iconClass: (addon.iconClass || "fas fa-cubes") + " fa-3x",
          document: addon.document,
        });
      }
      this.showAddonGallery();
    },

    getInstalledAddons: function() {
      this.installedAddons = {};
      var that = this;
      wagbyDesigner.sendRequest({
        url: "/addons/installed",
        options: {
          handleAs: "json",
        }
      }, function(addons) {
        that.setInstalledAddonInfo(addons);
      }, function(err) {
        return wagbyDesigner.jfcRes.MSG_ERROR_OPERATION_ADDON_GET;
      });
    },

    setInstalledAddonInfo: function(addons) {
      this.clearInstalledAddonInfo();
      for (var addonId in addons) {
        this.installedAddons[addonId] = addons[addonId];
        this.checkInstalledAddonVersion(addonId, addons[addonId]);
      }
    },

    clearInstalledAddonInfo: function() {
      var that = this;
      this.addonGalleryStore.query(function(addon) {
        that.clearAddonInfo(addon.id);
      });
    },

    clearAddonInfo: function(addonId) {
      var addonIconNode = this.getAddonIconNode(addonId);
      if (addonIconNode) {
        this.removeInstalledAddonVersion(addonIconNode);
        this.removeOldAddonIconClass(addonIconNode);
      }
    },

    checkInstalledAddonVersion: function(addonId, installedAddon) {
      this.setInstalledAddonVersion(addonId, installedAddon);
      this.setOldVersionAddonBlink(addonId, installedAddon);
    },

    getAddonIconNode: function(addonId) {
      return dom.byId(this.ADDONICON_ID_PREFIX + addonId);
    },

    setInstalledAddonVersion: function(addonId, installedAddon) {
      var addonIconNode = this.getAddonIconNode(addonId);
      if (addonIconNode) {
        domAttr.set(addonIconNode, "title", wagbyDesigner.jfcRes.TITLE_OPERATION_INSTALLEDADDON_VERSION + ":" + installedAddon.version);
      }
    },

    removeInstalledAddonVersion: function(addonIconNode) {
      domAttr.remove(addonIconNode, "title");
    },

    removeOldAddonIconClass: function(addonIconNode) {
      domClass.remove(addonIconNode, this.ADDONICON_UPDATE_CLASS);
    },

    setOldVersionAddonBlink: function(addonId, installedAddon) {
      var addonIconNode = this.getAddonIconNode(addonId);
      if (!addonIconNode) {
        return;
      }
      var addon = this.addonGalleryStore.get(addonId);
      if (addon.version === installedAddon.version) {
        this.removeOldAddonIconClass(addonIconNode);
      } else {
        domClass.add(addonIconNode, this.ADDONICON_UPDATE_CLASS);
      }
    },

    canInstallAddon: function(target) {
      var targetVersionArray = wagbyDesigner.getVersionArray(target);
      if (!targetVersionArray || targetVersionArray.length !== 3) {
        return false;
      }
      if (wagbyDesigner.wdMajorVersion < targetVersionArray[0]) {
        return false;
      }
      if (wagbyDesigner.wdMinorVersion > targetVersionArray[1]) {
        return true;
      }
      if (wagbyDesigner.wdMinorVersion === targetVersionArray[1] &&
          wagbyDesigner.wdTinyVersion >= targetVersionArray[2]) {
        return true;
      }
      return false;
    },

    clearAddonGalleryArea: function() {
      this.destroyAddonRadioButton();
      domConstruct.empty(this.addonGalleryArea);
    },

    showAddonGallery: function() {
      this.clearAddonGalleryArea();
      var that = this;
      this.addonGalleryStore.query(function(addon) {
        that.createAddonDom(addon);
      });
      addonGalleryStandby.hide();
    },

    getCursorStyle: function(addon) {
      if (!addon.canInstall) {
        return " cursor: default;";
      }
      return "";
    },

    getAddonDisabled: function(addon) {
      if (!addon.canInstall) {
        return "Disabled";
      }
      return "";
    },

    getAddonLabelTitle: function(addon) {
      if (addon.canInstall) {
        return wagbyDesigner.jfcRes.TITLE_OPERATION_ADDON_CHOOSE_ADDON;
      } else {
        return wagbyDesigner.jfcRes.TITLE_OPERATION_ADDON_CANNOT_INSTALL;
      }
    },

    createAddonDom: function(addon) {
      var template =
        '<div class="addon' + this.getAddonDisabled(addon) + '" ' + this.addonNodeSelectedAttrName + '="false">' +
          '<div class="info">' +
            '<input type="radio" id="addon_' + addon.id + '">' +
            '<label for="addon_' + addon.id + '" title="' + this.getAddonLabelTitle(addon) + '" style="margin: 0; padding: 0;' + this.getCursorStyle(addon) + '">' +
              '<i id="' + this.ADDONICON_ID_PREFIX + addon.id + '" class="' + addon.iconClass + '"></i>' +
              '<div>' +
                '<div class="name">' + addon.name + '</div>' +
                '<div class="version">' + wagbyDesigner.jfcRes.LABEL_OPERATION_ADDON_VERSION + ':' + addon.version + '</div>' +
              '</div>' +
            '</label>' +
          '</div>' +
          '<div id="addon_summary_' + addon.id + '" class="summary">' + addon.summary + '</div>' +
        '</div>';
      var addonBox = domConstruct.toDom(template);
      domConstruct.place(addonBox, this.addonGalleryArea);
      this.createAddonRadioButton(addon);
    },

    createAddonRadioButton: function(addon) {
      var radioButton = new RadioButton({
        checked: false,
        name: "addonId",
        style: "margin: 0; padding: 0; visibility: hidden; width: 0;",
        value: addon.id,
        disabled: !addon.canInstall,
      }, "addon_" + addon.id);
      radioButton.startup();
      // アドオンのUI部品のイベントをセットする
      this.setAddonUIEvent(radioButton, addon);
    },

    setAddonUIEvent: function(radioButton, addon) {
      // ラベルタグ押下でラジオボタンの選択をトグル式で切り替える
      if (addon.canInstall) {
        this.setLabelClickEvent(radioButton);
      }
      // ラジオボタンの選択状態変更イベントでアドオンの色を変更する
      var that = this;
      this.radioWatchHandles.push(radioButton.watch("checked", function(name, oldValue, value) {
        that.changeAddonBoxColor(radioButton, oldValue, value);
      }));
      // 概要部分を押下するとアドオンの詳細説明ダイアログを開くようにする
      this.setSummaryClickEvent(radioButton, addon);
    },

    setLabelClickEvent: function(radioButton) {
      var addonNode = radioButton.domNode.parentNode.parentNode;
      var labelNode = radioButton.domNode.nextSibling;
      var that = this;
      on(labelNode, "click", function(e) {
        baseEvent.stop(e);
        that.toggleRadioButton(radioButton, addonNode);
      });
    },

    isAddonSelected: function(addonNode) {
      return domAttr.get(addonNode, this.addonNodeSelectedAttrName) === "true";
    },

    toggleRadioButton: function(radioButton, addonNode) {
      if (this.isAddonSelected(addonNode)) {
        this.clearSelectedAddon();
      } else {
        radioButton.set("checked", true);
        addonAuthKey.set("value", "");
        this.setBtnAddonInstallDisabled(false);
        this.setBtnAddonUninstallDisabled(radioButton.value);
      }
    },

    clearSelectedAddon: function() {
      addonGalleryForm.reset();
      this.setBtnAddonInstallDisabled(true);
      this.setBtnAddonUninstallDisabled();
    },

    changeAddonBoxColor: function(radioButton, oldValue, value) {
      var addonNode = radioButton.domNode.parentNode.parentNode;
      var bgColor = this.addonDefaultColor;
      var borderColor = this.addonBoderDefaultColor;
      var selected = "false";
      if (value) {
        bgColor = this.addonSelectColor;
        borderColor = this.addonBoderSelectColor;
        selected = "true";
      }
      domStyle.set(addonNode, "backgroundColor", bgColor);
      domStyle.set(addonNode, "borderColor", borderColor);
      domAttr.set(addonNode, this.addonNodeSelectedAttrName, selected);
    },

    destroyAddonRadioButton: function() {
      array.forEach(this.radioWatchHandles, function(handle) {
        handle.unwatch();
      });
      query("input[type='radio'][name='addonId']", this.addonGalleryArea).forEach(function(node) {
        var widget = registry.byId(node.id);
        if (widget) {
          widget.destroyRecursive();
        }
      });
      this.radioWatchHandles = [];
    },

    setSummaryClickEvent: function(radioButton, addon) {
      if (!addon.document || addon.document === "") {
        return;
      }
      var summaryDom = radioButton.domNode.parentNode.nextSibling;
      domAttr.set(summaryDom, "title", wagbyDesigner.jfcRes.TITLE_OPERATION_ADDON_SHOW_DETAIL);
      var that = this;
      on(summaryDom, "click", function() {
        that.showAddonDocument(addon.document);
      });
    },

    showAddonDocument: function(document) {
      domStyle.set(this.addonDetailDialog.domNode, "visibility", "hidden");
      this.addonDetailDialog.set("href", document);
      var that = this;
      this.addonDetailDialog.show().then(function() {
        domStyle.set(that.addonDetailDialog.domNode, {
          top: "50px",
          visibility: "visible",
        });
        that.addonDetailDialog.okButton.focus();
      });
    },

    checkAddonAuthKey: function(selectedAddon) {
      var addon = this.addonGalleryStore.get(selectedAddon.addonId);
      if (addon.authKeyRequired) {
        this.addonAuthKeyInputDialog.show();
      } else {
        this.checkAddonConflict(selectedAddon);
      }
    },

    checkAddonConflict: function(selectedAddon) {
      if (this.installedAddons[selectedAddon.addonId]) {
        this.confirmOverwriteInstall(selectedAddon);
      } else {
        this.sendInstallData(selectedAddon);
      }
    },

    confirmOverwriteInstall: function(selectedAddon) {
      var that = this;
      var message = this.getOverwriteInstallMessage(selectedAddon.addonId);
      this.showConfirmDialog({
        'message': message,
        'onExecute': function() {
          that.sendInstallData(selectedAddon);
        },
        'onShow': function() {
          that.confirmDialog.cancelButton.focus();
        },
      });
    },

    getOverwriteInstallMessage: function(addonId) {
      var conflictModels = [];
      var models = this.installedAddons[addonId].models;
      for (var i=0,l=models.length; i<l; i++) {
        var model = wagbyDesigner.project.getModel(models[i]);
        conflictModels.push(model.getDescriptionWithId());
      }
      var message = wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_CONFIRM_OVERWRITE
        + this.getModelListMessage(conflictModels);
      return message;
    },

    sendInstallData: function(selectedAddon) {
      addonGalleryStandby.show();
      var that = this;
      wagbyDesigner.sendRequest({
        'url': "/addon",
        'options': {
          'method': "POST",
          'handleAs': "json",
          'data': selectedAddon,
        }
      }, function(data) {
        that.afterInstall(data);
      }, function(err) {
console.log("err: ", err);
        addonGalleryStandby.hide();
        var errorMessage = wagbyDesigner.jfcRes.MSG_ERROR_OPERATION_ADDON_INSTALL;
        if (err.response && err.response.status === 403) {
          errorMessage = wagbyDesigner.jfcRes.MSG_ERROR_OPERATION_ADDON_AUTHENTICATION_FAILED;
        }
        return errorMessage;
      });
    },

    afterInstall: function(data) {
      this.updateInstalledAddonsAndModels();
      addonGalleryStandby.hide();
      this.showModelIdDescription(
        data,
        wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_INSTALL_RESULT,
        wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_NOT_INSTALL
      );
    },

    confirmExecute: function(message, onExecuteFunc, button) {
      var selectedAddon = addonGalleryForm.getValues();
      if (!selectedAddon) {
        return;
      }
      var that = this;
      this.showConfirmDialog({
        'message': message,
        'onExecute': function() {
          onExecuteFunc(selectedAddon);
        },
        'onShow': function() {
          button.focus();
        },
      });
    },

    uninstallAddon: function() {
      var selectedAddon = addonGalleryForm.getValues();
      if (!selectedAddon) {
        return;
      }
      addonGalleryStandby.show();
      var that = this;
      var url = "/addon/" + selectedAddon.addonId;
      wagbyDesigner.sendRequest({
        'url': url,
        'options': {
          'method': "DELETE",
          'handleAs': "json",
        }
      }, function(data) {
        that.afterUninstall(data);
      }, function(err) {
console.log("err: ", err);
        addonGalleryStandby.hide();
        return wagbyDesigner.jfcRes.MSG_ERROR_OPERATION_ADDON_INSTALL;
      });
    },

    afterUninstall: function(data) {
      this.updateInstalledAddonsAndModels();
      addonGalleryStandby.hide();
      this.showModelIdDescription(
        data,
        wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_UNINSTALL_RESULT,
        wagbyDesigner.jfcRes.MSG_OPERATION_ADDON_NOT_UNINSTALL
      );
    },

    updateInstalledAddonsAndModels: function() {
      this.getInstalledAddons();
      this.clearSelectedAddon();
    },

    showModelIdDescription: function(modelIdDescriptionList, msg, errMsg) {
      if (!modelIdDescriptionList || modelIdDescriptionList.length === 0) {
        return errMsg;
      }
      var message = msg + this.getModelListMessage(modelIdDescriptionList);
      wagbyDesigner.showDialog(message);
    },

    getModelListMessage: function(modelList) {
      var message = "";
      if (!modelList) {
        return message;
      }
      for (var i=0,l=modelList.length; i<l; i++) {
        message = message + "- " + modelList[i] + "\n";
      }
      return message;
    },

  });
});
